@@ -1,7 +1,7 @@
 /* poppler-qt.h: qt interface to poppler
  * Copyright (C) 2005, Net Integration Technologies, Inc.
  * Copyright (C) 2005, 2007, Brad Hards <bradh@frogmouth.net>
- * Copyright (C) 2005-2015, 2017-2021, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2005-2015, 2017-2022, 2026, Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2005, Stefan Kebekus <stefan.kebekus@math.uni-koeln.de>
  * Copyright (C) 2006-2011, Pino Toscano <pino@kde.org>
  * Copyright (C) 2009 Shawn Rutledge <shawn.t.rutledge@gmail.com>
@@ -16,15 +16,23 @@
  * Copyright (C) 2012, 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
  * Copyright (C) 2013 Anthony Granger <grangeranthony@gmail.com>
  * Copyright (C) 2016 Jakub Alba <jakubalba@gmail.com>
- * Copyright (C) 2017, 2020 Oliver Sander <oliver.sander@tu-dresden.de>
+ * Copyright (C) 2017, 2020, 2021 Oliver Sander <oliver.sander@tu-dresden.de>
  * Copyright (C) 2017, 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
- * Copyright (C) 2018 Nelson Benítez León <nbenitezl@gmail.com>
+ * Copyright (C) 2018, 2021 Nelson Benítez León <nbenitezl@gmail.com>
  * Copyright (C) 2019 Jan Grulich <jgrulich@redhat.com>
  * Copyright (C) 2019 Alexander Volkov <a.volkov@rusbitech.ru>
  * Copyright (C) 2020 Philipp Knechtges <philipp-dev@knechtges.com>
  * Copyright (C) 2020 Katarina Behrens <Katarina.Behrens@cib.de>
  * Copyright (C) 2020 Thorsten Behrens <Thorsten.Behrens@CIB.de>
  * Copyright (C) 2020 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by Technische Universität Dresden
+ * Copyright (C) 2021 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>.
+ * Copyright (C) 2021 Mahmoud Khalil <mahmoudkhalil11@gmail.com>
+ * Copyright (C) 2021 Georgiy Sgibnev <georgiy@sgibnev.com>. Work sponsored by lab50.net.
+ * Copyright (C) 2022 Martin <martinbts@gmx.net>
+ * Copyright (C) 2023 Kevin Ottens <kevin.ottens@enioka.com>. Work sponsored by De Bortoli Wines
+ * Copyright (C) 2024 Pratham Gandhi <ppg.1382@gmail.com>
+ * Copyright (C) 2025, g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+ * Copyright (C) 2025 Stefan Brüns <stefan.bruens@rwth-aachen.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,6 +52,8 @@
 #ifndef __POPPLER_QT_H__
 #define __POPPLER_QT_H__
 
+#include <functional>
+
 #include "poppler-annotation.h"
 #include "poppler-link.h"
 #include "poppler-optcontent.h"
@@ -88,7 +98,7 @@ struct OutlineItemData;
 
     \since 0.16
 */
-typedef void (*PopplerDebugFunc)(const QString & /*message*/, const QVariant & /*closure*/);
+using PopplerDebugFunc = void (*)(const QString & /*message*/, const QVariant & /*closure*/);
 
 /**
     Set a new debug/error output function.
@@ -103,6 +113,30 @@ typedef void (*PopplerDebugFunc)(const QString & /*message*/, const QVariant & /
 */
 POPPLER_QT5_EXPORT void setDebugErrorFunction(PopplerDebugFunc debugFunction, const QVariant &closure);
 
+/**
+ * The various types of error strings.
+ *
+ * \since 25.07
+ */
+enum class ErrorStringType
+{
+    /**The string should be treated like a error code. It could be a hex code, a position in the poppler sources or something similar*/
+    ErrorCodeString,
+    /**The string should be treated as an advanced error message that can be shown to user */
+    UserString
+};
+
+/**
+ * Combination of an error data and type of error string
+ *
+ * \since 25.07
+ */
+struct ErrorString
+{
+    QVariant data;
+    ErrorStringType type;
+};
+
 /**
     Describes the physical location of text on a document page
 
@@ -202,7 +236,7 @@ public:
     /**
        Create a new font information container.
     */
-    FontInfo(const FontInfoData &fid);
+    explicit FontInfo(const FontInfoData &fid);
     /// \endcond
 
     /**
@@ -341,7 +375,7 @@ class POPPLER_QT5_EXPORT EmbeddedFile
 
 public:
     /// \cond PRIVATE
-    EmbeddedFile(EmbFile *embfile);
+    explicit EmbeddedFile(EmbFile *embfile);
     /// \endcond
 
     /**
@@ -411,7 +445,7 @@ public:
 
 private:
     Q_DISABLE_COPY(EmbeddedFile)
-    EmbeddedFile(EmbeddedFileData &dd);
+    explicit EmbeddedFile(EmbeddedFileData &dd);
 
     EmbeddedFileData *m_embeddedFile;
 };
@@ -461,7 +495,8 @@ public:
     enum TextLayout
     {
         PhysicalLayout, ///< The text is layouted to resemble the real page layout
-        RawOrderLayout ///< The text is returned without any type of processing
+        RawOrderLayout, ///< The text is returned without any type of processing
+        ReadingOrder, ///< The text is returned in reading order, \since 26.01
     };
 
     /**
@@ -531,7 +566,7 @@ public:
 
         \since 0.62
     */
-    typedef void (*RenderToImagePartialUpdateFunc)(const QImage & /*image*/, const QVariant & /*closure*/);
+    using RenderToImagePartialUpdateFunc = void (*)(const QImage & /*image*/, const QVariant & /*closure*/);
 
     /**
         Partial Update query renderToImage callback.
@@ -541,7 +576,7 @@ public:
 
         \since 0.62
     */
-    typedef bool (*ShouldRenderToImagePartialQueryFunc)(const QVariant & /*closure*/);
+    using ShouldRenderToImagePartialQueryFunc = bool (*)(const QVariant & /*closure*/);
 
     /**
        Render the page to a QImage using the current
@@ -602,7 +637,7 @@ public:
 
         \since 0.63
     */
-    typedef bool (*ShouldAbortQueryFunc)(const QVariant & /*closure*/);
+    using ShouldAbortQueryFunc = bool (*)(const QVariant & /*closure*/);
 
     /**
 Render the page to a QImage using the current
@@ -721,6 +756,9 @@ rather unexpected results.
        with coordinates given in points, i.e., 1/72th of an inch.
        If rect is null, all text on the page is given
 
+       \param textLayout the layouting that should be applied to the text.
+                         ReadingOrder is not supported with non null rect
+
        \since 0.16
     **/
     QString text(const QRectF &rect, TextLayout textLayout) const;
@@ -762,9 +800,12 @@ rather unexpected results.
         NoSearchFlags = 0x00000000, ///< since 0.63
         IgnoreCase = 0x00000001, ///< Case differences are ignored
         WholeWords = 0x00000002, ///< Only whole words are matched
-        IgnoreDiacritics = 0x00000004 ///< Diacritic differences (eg. accents, umlauts, diaeresis) are ignored. \since 0.73
-                                      ///< This option will have no effect if the search term contains characters which
-                                      ///< are not pure ascii.
+        IgnoreDiacritics = 0x00000004, ///< Diacritic differences (eg. accents, umlauts, diaeresis) are ignored. \since 0.73
+                                       ///< This option will have no effect if the search term contains characters which
+                                       ///< are not pure ascii.
+        AcrossLines = 0x00000008 ///< Allows to match on text spanning from end of a line to the next line.
+                                 ///< It won't match on text spanning more than two lines. Automatically ignores hyphen
+                                 ///< at end of line, and allows whitespace in search term to match on newline. \since 21.05.0
     };
     Q_DECLARE_FLAGS(SearchFlags, SearchFlag)
 
@@ -811,6 +852,9 @@ rather unexpected results.
     /**
        Returns a list of all occurrences of the specified text on the page.
 
+       if SearchFlags::AcrossLines is given in \param flags, then rects may just
+       be parts of the text itself if it's split between multiple lines.
+
        \param text the text to search
        \param flags the flags to consider during matching
        \param rotate the rotation to apply for the search order
@@ -1074,7 +1118,7 @@ public:
     QVector<OutlineItem> children() const;
 
 private:
-    OutlineItem(OutlineItemData *data);
+    explicit OutlineItem(OutlineItemData *data);
     OutlineItemData *m_data;
 };
 
@@ -1329,6 +1373,11 @@ public:
        This function can return nullptr if for some reason the page can't be properly parsed.
 
        \param index the page number index
+
+       \warning The Page object returned by this method internally stores a pointer
+       to the document that it was created from.  This pointer will go stale if you
+       delete the Document object.  Therefore the Document object needs to be kept alive
+       as long as you want to use the Page object.
     */
     Page *page(int index) const;
 
@@ -1633,9 +1682,27 @@ QString subject = m_doc->info("Subject");
        \param minor an optional pointer to a variable where store the
        "minor" number of the version
 
+       \deprecated Will be removed in the Qt6 interface.  Use the method
+       returning a PdfVersion object instead!
+
        \since 0.12
     */
-    void getPdfVersion(int *major, int *minor) const;
+    Q_DECL_DEPRECATED void getPdfVersion(int *major, int *minor) const;
+
+    /** \brief The version specification of a pdf file */
+    struct PdfVersion
+    {
+        int major;
+        int minor;
+    };
+
+    /**
+       The version of the PDF specification that the document
+       conforms to
+
+       \since 21.08
+    */
+    PdfVersion getPdfVersion() const;
 
     /**
        The fonts within the PDF document.
@@ -1833,6 +1900,13 @@ QString subject = m_doc->info("Subject");
     */
     OptContentModel *optionalContentModel();
 
+    /**
+       Resets the form with the details contained in the \p link.
+
+       \since 24.07
+    */
+    void applyResetFormsLink(const LinkResetForm &link);
+
     /**
        Document-level JavaScript scripts.
 
@@ -1843,6 +1917,31 @@ QString subject = m_doc->info("Subject");
     */
     QStringList scripts() const;
 
+    /**
+      Describes the flags for additional document actions i.e.
+      for executing document scripts at different events.
+      This flag is used by additionalAction method to return the
+      particular Link.
+
+      \since 24.07
+    */
+    enum DocumentAdditionalActionsType
+    {
+        CloseDocument, ///< Performed before closing the document
+        SaveDocumentStart, ///< Performed before saving the document
+        SaveDocumentFinish, ///< Performed after saving the document
+        PrintDocumentStart, ///< Performed before printing the document
+        PrintDocumentFinish, ///< Performed after printing the document
+    };
+
+    /**
+      Returns the additional action of the given @p type for the document or
+      @c 0 if no action has been defined.
+
+      \since 24.07
+    */
+    Link *additionalAction(DocumentAdditionalActionsType type) const;
+
     /**
        The PDF identifiers.
 
@@ -1881,6 +1980,21 @@ QString subject = m_doc->info("Subject");
     */
     QVector<FormFieldSignature *> signatures() const;
 
+    /**
+     Returns whether the document's XRef table has been reconstructed or not
+
+     \since 21.06
+    */
+    bool xrefWasReconstructed() const;
+
+    /**
+     Sets the document's XRef reconstruction callback, so whenever a XRef table
+     reconstruction happens the callback will get triggered.
+
+     \since 21.06
+    */
+    void setXRefReconstructedCallback(const std::function<void()> &callback);
+
     /**
        Destructor.
     */
@@ -1891,7 +2005,7 @@ private:
 
     DocumentData *m_doc;
 
-    Document(DocumentData *dataA);
+    explicit Document(DocumentData *dataA);
 };
 
 class BaseConverterPrivate;
@@ -1947,7 +2061,7 @@ public:
 
 protected:
     /// \cond PRIVATE
-    BaseConverter(BaseConverterPrivate &dd);
+    explicit BaseConverter(BaseConverterPrivate &dd);
     Q_DECLARE_PRIVATE(BaseConverter)
     BaseConverterPrivate *d_ptr;
     /// \endcond
@@ -1988,7 +2102,8 @@ public:
         StrictMargins = 0x00000002,
         ForceRasterization = 0x00000004,
         PrintToEPS = 0x00000008, ///< Output EPS instead of PS \since 0.20
-        HideAnnotations = 0x00000010 ///< Don't print annotations \since 0.20
+        HideAnnotations = 0x00000010, ///< Don't print annotations \since 0.20
+        ForceOverprintPreview = 0x00000020 ///< Force rasterized overprint preview during conversion \since 23.09
     };
     Q_DECLARE_FLAGS(PSOptions, PSOption)
 
@@ -2059,6 +2174,16 @@ public:
     */
     void setStrictMargins(bool strictMargins);
 
+    /**
+      Defines if the page will be rasterized to an image with overprint
+      preview enabled before printing.
+
+      Defaults to false
+
+      \since 23.09
+    */
+    void setForceOverprintPreview(bool forceOverprintPreview);
+
     /** Defines if the page will be rasterized to an image before printing. Defaults to false */
     void setForceRasterize(bool forceRasterize);
 
@@ -2093,7 +2218,7 @@ private:
     Q_DECLARE_PRIVATE(PSConverter)
     Q_DISABLE_COPY(PSConverter)
 
-    PSConverter(DocumentData *document);
+    explicit PSConverter(DocumentData *document);
 };
 
 /**
@@ -2137,7 +2262,8 @@ public:
      *  - rect for the signature annotation
      *  - text that will be shown inside the rect
      *  - font size and color
-     *  - border and background color
+     *  - border width and color
+     *  - background color
      * \since 21.01
      */
     class POPPLER_QT5_EXPORT NewSignatureData
@@ -2163,12 +2289,50 @@ public:
         QString signatureText() const;
         void setSignatureText(const QString &text);
 
+        /**
+         * If this text is not empty, the signature representation
+         * will split in two, with this text on the left and signatureText
+         * on the right
+         *
+         * \since 21.06
+         */
+        QString signatureLeftText() const;
+        void setSignatureLeftText(const QString &text);
+
+        /**
+         * Signature's property Reason.
+         *
+         * Default: an empty string.
+         *
+         * \since 21.10
+         */
+        QString reason() const;
+        void setReason(const QString &reason);
+
+        /**
+         * Signature's property Location.
+         *
+         * Default: an empty string.
+         *
+         * \since 21.10
+         */
+        QString location() const;
+        void setLocation(const QString &location);
+
         /**
          * Default: 10
          */
         double fontSize() const;
         void setFontSize(double fontSize);
 
+        /**
+         * Default: 20
+         *
+         * \since 21.06
+         */
+        double leftFontSize() const;
+        void setLeftFontSize(double fontSize);
+
         /**
          * Default: red
          */
@@ -2181,6 +2345,16 @@ public:
         QColor borderColor() const;
         void setBorderColor(const QColor &color);
 
+        /**
+         * border width in points
+         *
+         * Default: 1.5
+         *
+         * \since 21.05
+         */
+        double borderWidth() const;
+        void setBorderWidth(double width);
+
         /**
          * Default: QColor(240, 240, 240)
          */
@@ -2193,6 +2367,37 @@ public:
         QString fieldPartialName() const;
         void setFieldPartialName(const QString &name);
 
+        /**
+         * Document owner password (needed if the document that is being signed is password protected)
+         *
+         * Default: no password
+         *
+         * \since 22.02
+         */
+        QByteArray documentOwnerPassword() const;
+        void setDocumentOwnerPassword(const QByteArray &password);
+
+        /**
+         * Document user password (needed if the document that is being signed is password protected)
+         *
+         * Default: no password
+         *
+         * \since 22.02
+         */
+        QByteArray documentUserPassword() const;
+        void setDocumentUserPassword(const QByteArray &password);
+
+        /**
+         * Filesystem path to an image file to be used as background
+         * image for the signature annotation widget.
+         *
+         * Default: empty
+         *
+         * \since 22.02
+         */
+        QString imagePath() const;
+        void setImagePath(const QString &path);
+
     private:
         struct NewSignatureDataPrivate;
         NewSignatureDataPrivate *const d;
@@ -2209,13 +2414,43 @@ public:
     */
     bool sign(const NewSignatureData &data);
 
+    /**
+     * \since 25.07
+     *
+     * BIC/SIC: Merge with SigningResult in poppler-converter and in poppler-annotation and poppler-form
+     */
+    enum SigningResult
+    {
+        SigningSuccess, ///< No error
+        FieldAlreadySigned, ///< Trying to sign a field that is already signed
+        GenericSigningError, ///< Unclassified error
+        InternalError, ///< Unexpected error, likely a bug in poppler
+        KeyMissing, ///< Key not found (Either the input key is not from the list or the available keys has changed underneath)
+        WriteFailed, ///< Write failed (permissions, faulty disk, ...)
+        UserCancelled, ///< User cancelled the process
+        BadPassphrase, ///< User entered bad passphrase
+    };
+
+    /**
+     * The last signing result, mostly relevant if \ref sign returns false
+     * \since 25.07
+     */
+    SigningResult lastSigningResult() const;
+
+    /**
+     * A string with a string that might offer more details of the signing result failure
+     * \note the string here is likely not super useful for end users, but might give more details to a trained supporter / bug triager
+     * \since 25.07
+     */
+    ErrorString lastSigningErrorDetails() const;
+
     bool convert() override;
 
 private:
     Q_DECLARE_PRIVATE(PDFConverter)
     Q_DISABLE_COPY(PDFConverter)
 
-    PDFConverter(DocumentData *document);
+    explicit PDFConverter(DocumentData *document);
 };
 
 /**
@@ -2278,7 +2513,7 @@ public:
     };
 
     /// \cond PRIVATE
-    SoundObject(Sound *popplersound);
+    explicit SoundObject(Sound *popplersound);
     /// \endcond
 
     ~SoundObject();
@@ -2389,7 +2624,7 @@ public:
 
 private:
     /// \cond PRIVATE
-    MovieObject(AnnotMovie *ann);
+    explicit MovieObject(AnnotMovie *ann);
     /// \endcond
 
     Q_DISABLE_COPY(MovieObject)
